;;; navi2ch-board-misc.el --- Miscellaneous Functions for Navi2ch Board Mode 

;; Copyright (C) 2001 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: 2ch, network

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'cl)
(require 'navi2ch-article)

(defvar navi2ch-board-window-height 10)
(defvar navi2ch-board-buffer-name "*navi2ch board*")
(defvar navi2ch-bm-subject-width 50)
(defvar navi2ch-bm-mode-map nil)
(unless navi2ch-bm-mode-map
  (setq navi2ch-bm-mode-map (make-sparse-keymap))
  (define-key navi2ch-bm-mode-map "\r" 'navi2ch-bm-select-article)
  (define-key navi2ch-bm-mode-map [mouse-2] 'navi2ch-bm-mouse-select)
  (define-key navi2ch-bm-mode-map " "
    'navi2ch-bm-select-article-or-scroll-up)
  (define-key navi2ch-bm-mode-map "o" 'navi2ch-bm-display-article)
  (define-key navi2ch-bm-mode-map "i" 'navi2ch-bm-fetch-article)
  (define-key navi2ch-bm-mode-map [del]
    'navi2ch-bm-select-article-or-scroll-down)
  (define-key navi2ch-bm-mode-map [backspace]
    'navi2ch-bm-select-article-or-scroll-down)
  (define-key navi2ch-bm-mode-map "n" 'next-line)
  (define-key navi2ch-bm-mode-map "p" 'previous-line)
  (define-key navi2ch-bm-mode-map "g" 'navi2ch-bm-goto-board)
  (define-key navi2ch-bm-mode-map "q" 'navi2ch-bm-exit)
  (define-key navi2ch-bm-mode-map "S" 'navi2ch-bm-sort)
  (define-key navi2ch-bm-mode-map ">" 'end-of-buffer)
  (define-key navi2ch-bm-mode-map "<" 'beginning-of-buffer)
  (define-key navi2ch-bm-mode-map "t" 'navi2ch-toggle-offline)
  (define-key navi2ch-bm-mode-map "1" 'navi2ch-one-pain)
  (define-key navi2ch-bm-mode-map "3" 'navi2ch-three-pain)
  (define-key navi2ch-bm-mode-map "\C-c\C-f" 'navi2ch-article-find-file)

  ;; mark command
  (define-key navi2ch-bm-mode-map "*" 'navi2ch-bm-mark)
  (define-key navi2ch-bm-mode-map "u" 'navi2ch-bm-unmark)
  (define-key navi2ch-bm-mode-map "mr" 'navi2ch-bm-mark-region)
  (define-key navi2ch-bm-mode-map "ma" 'navi2ch-bm-mark-all)
  (define-key navi2ch-bm-mode-map "mo" 'navi2ch-bm-display-mark-article)
  (define-key navi2ch-bm-mode-map "mi" 'navi2ch-bm-fetch-mark-article)
  (define-key navi2ch-bm-mode-map "mm" 'navi2ch-bm-mark-marks))

(defvar navi2ch-bm-get-property-function nil
  "$B0z?t$O(B POINT")
(defvar navi2ch-bm-set-property-function nil
  "$B0z?t$O(B BEGIN END ITEM")
(defvar navi2ch-bm-get-board-function nil
  "$B0z?t$O(B ITEM")
(defvar navi2ch-bm-get-article-function nil
  "$B0z?t$O(B ITEM")
;; (defvar navi2ch-bm-get-subject-function nil
;;   "$B0z?t$O(B ITEM")
(defvar navi2ch-bm-exit-function nil)

(defmacro navi2ch-bm-set-func (sym val)
  `(let ((val-str (symbol-name ',val))
         (sym-str (symbol-name ,sym))
         func-str)
     (when (string-match "navi2ch-bm-\\(.+\\)-function" val-str)
       (setq func-str (format "%s-%s" sym-str (match-string 1 val-str)))
       (setq ,val (intern func-str)))))
  
(defun navi2ch-bm-setup (prefix)
  (navi2ch-bm-set-func prefix navi2ch-bm-get-property-function)
  (navi2ch-bm-set-func prefix navi2ch-bm-set-property-function)
  (navi2ch-bm-set-func prefix navi2ch-bm-get-board-function)
  (navi2ch-bm-set-func prefix navi2ch-bm-get-article-function)
  ;; (navi2ch-bm-set-func prefix navi2ch-bm-get-subject-function)
  (navi2ch-bm-set-func prefix navi2ch-bm-exit-function))

(defun navi2ch-bm-select-board (board &optional force)
  (switch-to-buffer (get-buffer-create navi2ch-board-buffer-name))
  (let ((type (cdr (assq 'type board))))
    (cond ((eq type 'bookmark)
           (navi2ch-bookmark))
          ((eq type 'articles)
           (navi2ch-articles))
          (t
           (navi2ch-board-select-board board force)))))

(defsubst navi2ch-bm-set-property (begin end item)
  (funcall navi2ch-bm-set-property-function begin end item)
  (put-text-property begin end 'mouse-face 'highlight))

(defsubst navi2ch-bm-insert-subject (item number subject other)
  (let ((alist '((cache . "C") (view . "V")))
        (article (funcall navi2ch-bm-get-article-function item))
        (board (funcall navi2ch-bm-get-board-function item))
        (point (point)))
    (insert (format "%3d %s %s%s%s\n"
                    number
                    (or (cdr (assq (navi2ch-article-check-cached
                                    board article)
                                   alist)) " ")
                    subject
                    (make-string (max (- navi2ch-bm-subject-width
                                         (string-width subject))
                                      1)
                                 ? )
                    other))
    (navi2ch-bm-set-property point (1- (point)) item)))

(defun navi2ch-bm-exit ()
  (interactive)
  (dolist (x (navi2ch-article-buffer-list))
    (when x
      (delete-windows-on x)))
  (funcall navi2ch-bm-exit-function)
  (when (get-buffer navi2ch-board-buffer-name)
    (delete-windows-on navi2ch-board-buffer-name)
    (bury-buffer navi2ch-board-buffer-name))
  (when navi2ch-list-buffer-name
    (let ((win (get-buffer-window navi2ch-list-buffer-name)))
      (if win
	  (select-window win)
	(switch-to-buffer navi2ch-list-buffer-name)))))

(defun navi2ch-bm-goto-state-column ()
  (beginning-of-line)
  (forward-char 4))

(defun navi2ch-bm-insert-state (item state)
  ;; (setq article (navi2ch-put-alist 'cache 'view article))
  (navi2ch-bm-goto-state-column)
  (delete-char 1)
  (insert state)
  (navi2ch-bm-set-property (1- (point)) (point) item))
  
(defun navi2ch-bm-select-article (&optional force)
  (interactive "P")
  (let* ((item (funcall navi2ch-bm-get-property-function (point)))
         (article (funcall navi2ch-bm-get-article-function item))
         (board (funcall navi2ch-bm-get-board-function item))
         (buf (current-buffer)))
    (if article
        (progn
          (dolist (x (navi2ch-article-buffer-list))
            (when x
              (delete-windows-on x)))
          (split-window-vertically navi2ch-board-window-height)
          (other-window 1)
          (let (state)
            (setq state (navi2ch-article-view-article
                         board article force))
            (save-excursion
              (set-buffer buf)
              (let ((buffer-read-only nil))
                (when state
                  (navi2ch-bm-insert-state item "V"))))))
      (message "can't select this line!"))))

;; (defun navi2ch-bm-search-line (board article)
;;   (let ((uri (cdr (assq 'uri board)))
;;         (artid (cdr (assq 'artid artid))))
;;     (catch 'loop
;;       (while (not (eobp))
;;         (let ((item (funcall navi2ch-bm-get-property-function (point))))
;;           (when (and (string=
;;                       (cdr (assq
;;                             'uri
;;                             (funcall navi2ch-bm-get-board-function item)))
;;                       uri)
;;                      (string=
;;                       (cdr (assq
;;                             'artid
;;                             (funcall navi2ch-bm-get-article-function item)))
;;                       artid))
;;             (throw 'loop (point))))
;;         (forward-line)))))

;; (defun test ()
;;   (save-excursion
;;     (set-buffer navi2ch-board-buffer-name)
;;     (goto-char (point-min))
;;     (let (list)
;;       (while (not (eobp))
;;         (let ((article (funcall
;;                         navi2ch-bm-get-article-function
;;                         (funcall navi2ch-bm-get-property-function (point))))
;;               (str (buffer-substring (point)
;;                                      (progn (forward-line) (point)))))
;;           (setq list (cons (cons str
;;                                  article)
;;                            list))))
;;       (nreverse list))))

(defun navi2ch-bm-display-article (&optional force)
  (interactive "P")
  (let ((win (selected-window)))
    (navi2ch-bm-select-article force)
    (select-window win)))

(defun navi2ch-bm-fetch-article (&optional force)
  (interactive "P")
  (let* ((item (funcall navi2ch-bm-get-property-function (point)))
         (board (funcall navi2ch-bm-get-board-function item))
         (article (funcall navi2ch-bm-get-article-function item))
         state)
    (if article
        (progn
          (setq state (navi2ch-article-fetch-article board article))
          (when state
            (let ((buffer-read-only nil))
              (save-excursion
                (navi2ch-bm-insert-state item "U")))))
      (message "can't select this line!"))))
    
(defun navi2ch-bm-select-article-or-scroll (way &optional force)
  (let ((article (funcall navi2ch-bm-get-article-function
                          (funcall navi2ch-bm-get-property-function
                                   (point)))))
    (if (and (navi2ch-article-current-buffer)
             (string= (cdr (assq 'artid article))
                      (save-excursion
                        (set-buffer (navi2ch-article-current-buffer))
                        (cdr (assq 'artid navi2ch-article-current-article))))
             (get-buffer-window (navi2ch-article-current-buffer)))
        (let ((win (selected-window)))
          (select-window
           (get-buffer-window (navi2ch-article-current-buffer)))
          (condition-case nil
              (cond
               ((eq way 'up)
                (navi2ch-article-scroll-up))
               ((eq way 'down)
                (navi2ch-article-scroll-down)))
            (error nil))
          (select-window win))
      (navi2ch-bm-select-article force))))

(defun navi2ch-bm-select-article-or-scroll-up (&optional force)
  (interactive "P")
  (navi2ch-bm-select-article-or-scroll 'up force))

(defun navi2ch-bm-select-article-or-scroll-down (&optional force)
  (interactive "P")
  (navi2ch-bm-select-article-or-scroll 'down force))

(defun navi2ch-bm-mouse-select (e)
  (interactive "e")
  (mouse-set-point e)
  (save-excursion
    (beginning-of-line)
    (navi2ch-bm-select-article)))

(defun navi2ch-bm-goto-mark-column ()
  (navi2ch-bm-goto-state-column)
  (forward-char 1))

(defun navi2ch-bm-mark-subr (mark)
  (let ((item (funcall navi2ch-bm-get-property-function (point))))
    (when item
      (save-excursion
        (let ((buffer-read-only nil))
          (navi2ch-bm-goto-mark-column)
          (delete-char 1)
          (insert mark)
          (navi2ch-bm-set-property (1- (point)) (point) item))))))

(defun navi2ch-bm-mark ()
  (interactive)
  (navi2ch-bm-mark-subr "*"))

(defun navi2ch-bm-unmark ()
  (interactive)
  (navi2ch-bm-mark-subr " "))

(defun navi2ch-bm-exec-subr (func)
  (save-excursion
    (goto-char (point-min))
    (while (not (eobp))
      (navi2ch-bm-goto-mark-column)
      (when (looking-at "\\*")
        (funcall func)
        (navi2ch-bm-unmark))
      (forward-line))))

(defun navi2ch-bm-display-mark-article ()
  (interactive)
  (navi2ch-bm-exec-subr 'navi2ch-bm-display-article))

(defun navi2ch-bm-fetch-mark-article ()
  (interactive)
  (navi2ch-bm-exec-subr 'navi2ch-bm-fetch-article))

(defun navi2ch-bm-goto-board ()
  (interactive)
  (navi2ch-list-goto-board
   (funcall navi2ch-bm-get-board-function
            (funcall navi2ch-bm-get-property-function
                     (point)))))

(defun navi2ch-bm-mark-region-subr (begin end mark)
  (save-excursion
    (save-restriction
      (narrow-to-region begin end)
      (goto-char (point-min))
      (while (not (eobp))
        (navi2ch-bm-mark-subr mark)
        (forward-line)))))

(defun navi2ch-bm-mark-region (begin end &optional arg)
  (interactive "r\nP")
  (navi2ch-bm-mark-region-subr begin end (if arg " " "*")))

(defun navi2ch-bm-mark-all (&optional arg)
  (interactive "P")
  (navi2ch-bm-mark-region (point-min) (point-max) arg))

(defun navi2ch-bm-mark-marks (mark &optional arg)
  (interactive "cinput mark: \nP")
  (save-excursion
    (goto-char (point-min))
    (let ((rep (format "%c" (upcase mark))))
      (while (not (eobp))
        (navi2ch-bm-goto-state-column)
        (when (looking-at rep)
          (navi2ch-bm-mark-subr (if arg " " "*")))
        (forward-line)))))

(defun navi2ch-bm-sort-subr (rev start-key-fun end-key-fun)
  (let ((buffer-read-only nil))
    (save-excursion
      (goto-char (point-min))
      (sort-subr rev 'forward-line 'end-of-line
                 start-key-fun end-key-fun))))

(defun navi2ch-bm-sort-by-number (rev)
  (interactive "P")
  (navi2ch-bm-sort-subr
   rev
   'beginning-of-line
   'navi2ch-bm-goto-state-column))

(defun navi2ch-bm-sort-by-state (rev)
  (interactive "P")
  (navi2ch-bm-sort-subr
   (not rev)
   'navi2ch-bm-goto-state-column
   'forward-char))

(defun navi2ch-bm-goto-other-column ()
  (let ((sbj (cdr
              (assq 'subject
                    (funcall
                     navi2ch-bm-get-article-function
                     (funcall navi2ch-bm-get-property-function (point)))))))
    (navi2ch-bm-goto-mark-column)
    (forward-char 1)
    (when (and (not (string= sbj ""))
               (search-forward sbj))
      (goto-char (match-end 0)))
    (skip-chars-forward " ")))

(defun navi2ch-bm-sort-by-subject (rev)
  (interactive "P")
  (navi2ch-bm-sort-subr
   rev
   (lambda ()
     (navi2ch-bm-goto-mark-column)
     (forward-char 1))
   'navi2ch-bm-goto-other-column))

(defun navi2ch-bm-sort-by-other (rev)
  (interactive "P")
  (navi2ch-bm-sort-subr
   rev
   (lambda ()
     (navi2ch-bm-goto-other-column)
     nil) ; end-key-fun $B$r8F$P$;$k$K$O(B nil $B$,M_$7$$$i$7$$!#$O$^$C$?(B($B5c(B)$B!#(B
   'end-of-line))

(defun navi2ch-bm-sort (&optional arg)
  (interactive "P")
  (message "Sort by n)umber s)tate t)itle o)ther?")
  (let ((ch (read-char)))
    (funcall
     (cond ((eq ch ?n) 'navi2ch-bm-sort-by-number)
           ((eq ch ?s) 'navi2ch-bm-sort-by-state)
           ((eq ch ?t) 'navi2ch-bm-sort-by-subject)
           ((eq ch ?o) 'navi2ch-bm-sort-by-other))
     arg)))
  
(provide 'navi2ch-board-misc)

;;; navi2ch-board-misc.el ends here