;;; navi2ch.el --- Navigator for 2ch for Emacsen

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'navi2ch-util)
(require 'navi2ch-net)
(require 'navi2ch-list)
(require 'navi2ch-board)
(require 'navi2ch-article)
(require 'navi2ch-message)
(require 'navi2ch-bookmark)
(require 'navi2ch-articles)
(require 'navi2ch-board-misc)
(require 'cl)

(defvar navi2ch-offline nil
  "$B%*%U%i%$%s%b!<%I$+$I$&$+(B")

(defvar navi2ch-version "1.0")
(defvar navi2ch-init nil)

(defun navi2ch (&optional arg)
  "Navigator for 2ch for Emacs"
  (interactive "P")
  (unless navi2ch-init
    (add-hook 'kill-emacs-hook 'navi2ch-save-status)
    (if arg (setq navi2ch-offline (not navi2ch-offline)))
    (navi2ch-bookmark-load-info)
    (setq navi2ch-init t)
    (run-hooks 'navi2ch-hook))
  (navi2ch-list))

(defun navi2ch-save-status ()
  "list, board, article $B$N>uBV$rJ]B8$9$k(B"
  (interactive)
  (dolist (x (navi2ch-article-buffer-list))
    (save-excursion
      (set-buffer x)
      (navi2ch-article-save-number)
      (navi2ch-article-save-info)))
  (navi2ch-board-save-info)
  (navi2ch-list-save-info)
  (navi2ch-bookmark-save-info))
  
(defun navi2ch-exit (&optional suspend)
  "navi2ch $B$r=*N;$9$k(B
SUSPEND $B$,(B non-nil $B$J$i(B buffer $B$r>C$5$J$$(B"
  (interactive)
  (navi2ch-save-status)
  (dolist (x (append
              (list
               (get-buffer navi2ch-list-buffer-name)
               (get-buffer navi2ch-board-buffer-name))
              (navi2ch-article-buffer-list)))
    (when x
      (delete-windows-on x)
      (if suspend
          (bury-buffer x)
        (kill-buffer x))))
  (unless suspend
    (setq navi2ch-init nil)
    (remove-hook 'kill-emacs-hook 'navi2ch-save-status)))

(defun navi2ch-suspend ()
  "navi2ch $B$r0l;~E*$K=*N;$9$k(B"
  (interactive)
  (navi2ch-exit 'suspend))

(defun navi2ch-toggle-offline ()
  (interactive)
  (message (if navi2ch-offline
               "online"
             "offline"))
  (setq navi2ch-offline (not navi2ch-offline)))

(defun navi2ch-three-pain ()
  (interactive)
  (delete-other-windows)
  (let ((list-buf (get-buffer navi2ch-list-buffer-name))
         (board-buf (get-buffer navi2ch-board-buffer-name))
         (art-buf (navi2ch-article-current-buffer))
         buf)
    (if (or (eq (current-buffer) list-buf)
            (eq (current-buffer) board-buf)
            (eq (current-buffer) art-buf))
        (setq buf (current-buffer))
      (setq buf list-buf))
    (when list-buf
      (switch-to-buffer navi2ch-list-buffer-name)
      (when board-buf
        (split-window-horizontally navi2ch-list-window-width)
        (other-window 1)))
    (when board-buf
      (switch-to-buffer navi2ch-board-buffer-name)
      (when art-buf
        (split-window-vertically navi2ch-board-window-height)
        (other-window 1)))
    (when art-buf
      (switch-to-buffer art-buf))
    (select-window (get-buffer-window buf))))

(defun navi2ch-one-pain ()
  (interactive)
  (let ((list-buf (get-buffer navi2ch-list-buffer-name))
        (board-buf (get-buffer navi2ch-board-buffer-name))
        (art-buf (navi2ch-article-current-buffer)))
    (if (> (count-windows) 1)
        (delete-other-windows)
      (delete-other-windows)
      (switch-to-buffer
       (cond ((eq (current-buffer) list-buf)
              (or board-buf art-buf list-buf))
             ((eq (current-buffer) board-buf)
              (or art-buf list-buf board-buf))
             ((eq (current-buffer) art-buf)
              (or list-buf board-buf art-buf))
             (t
              (or list-buf board-buf art-buf (current-buffer))))))))

(defun navi2ch-save-info (file info)
  (let ((dir (file-name-directory file)))
    (unless (file-exists-p dir)
      (make-directory dir t)))
  (with-temp-file file
    (let ((standard-output (current-buffer)))
      (prin1 info))))

(defun navi2ch-load-info (file)
  (when (file-exists-p file)
    (with-temp-buffer
      (insert-file-contents file)
      (car (read-from-string (buffer-string))))))

(provide 'navi2ch)

;;; navi2ch.el ends here